=== modified file 'branchview/graphcell.py'
--- branchview/graphcell.py	2008-01-13 14:12:49 +0000
+++ branchview/graphcell.py	2008-01-21 12:31:58 +0000
@@ -38,6 +38,10 @@
                           "revision node instruction",
                           gobject.PARAM_WRITABLE
                         ),
+        "tags":         ( gobject.TYPE_PYOBJECT, "tags",
+                          "list of tags associated with the node",
+                          gobject.PARAM_WRITABLE
+                        ),
         "in-lines":     ( gobject.TYPE_PYOBJECT, "in-lines",
                           "instructions to draw lines into the cell",
                           gobject.PARAM_WRITABLE
@@ -52,6 +56,8 @@
         """Set properties from GObject properties."""
         if property.name == "node":
             self.node = value
+        elif property.name == "tags":
+            self.tags = value
         elif property.name == "in-lines":
             self.in_lines = value
         elif property.name == "out-lines":
@@ -168,8 +174,10 @@
 
         self.set_colour(ctx, colour, 0.5, 1.0)
         ctx.fill()
+
+        self.render_tags(ctx, widget.create_pango_context(), cell_area, box_size)
     
-    def render_line (self, ctx, cell_area, box_size, mid, height, start, end, colour):
+    def render_line(self, ctx, cell_area, box_size, mid, height, start, end, colour):
         if start is None:
             x = cell_area.x + box_size * end + box_size / 2
             ctx.move_to(x, mid + height / 3)
@@ -202,4 +210,36 @@
                 
         self.set_colour(ctx, colour, 0.0, 0.65)
         ctx.stroke()
-        
\ No newline at end of file
+
+    def render_tags(self, ctx, pango_ctx, cell_area, box_size):
+        (column, colour) = self.node
+        tag_layout = pango.Layout(pango_ctx)
+
+        for tag_idx, tag in enumerate(self.tags):
+            tag_layout.set_text(tag)
+            text_width, text_height = tag_layout.get_pixel_size()
+
+            x0 = cell_area.x + box_size * (column + 1.3)
+            y0 = cell_area.y + \
+                    cell_area.height * (tag_idx + 1) / (len(self.tags) + 1) - \
+                    text_height / 2
+
+            # Draw the tag border
+            ctx.move_to(x0 - box_size / 3, y0 + text_height / 2)
+            ctx.line_to(x0, y0)
+            ctx.line_to(x0 + text_width, y0)
+            ctx.line_to(x0 + text_width, y0 + text_height)
+            ctx.line_to(x0, y0 + text_height)
+            ctx.line_to(x0 - box_size / 3, y0 + text_height / 2)
+
+            self.set_colour(ctx, colour, 0.0, 0.5)
+            ctx.stroke_preserve()
+
+            self.set_colour(ctx, colour, 0.5, 1.0)
+            ctx.fill()
+
+            # Draw the tag text
+            self.set_colour(ctx, 0.0, 0.0, 0.0)
+            ctx.move_to(x0, y0)
+            ctx.show_layout(tag_layout)
+

=== modified file 'branchview/treemodel.py'
--- branchview/treemodel.py	2008-01-13 14:12:49 +0000
+++ branchview/treemodel.py	2008-01-21 12:31:58 +0000
@@ -25,22 +25,24 @@
 TIMESTAMP = 7
 REVISION = 8
 PARENTS = 9
-CHILDREN = 10
+CHILDREN = 10
+TAGS = 11
 
 class TreeModel(gtk.GenericTreeModel):
 
     
-    def __init__ (self, repository, line_graph_data):
+    def __init__ (self, branch, line_graph_data):
         gtk.GenericTreeModel.__init__(self)
-        self.revisions = {}
-        self.repository = repository
+        self.revisions = {}
+        self.branch = branch
+        self.repository = branch.repository
         self.line_graph_data = line_graph_data
     
     def on_get_flags(self):
         return gtk.TREE_MODEL_LIST_ONLY
     
     def on_get_n_columns(self):
-        return 11
+        return 12
     
     def on_get_column_type(self, index):
         if index == REVID: return gobject.TYPE_STRING
@@ -53,7 +55,8 @@
         if index == TIMESTAMP: return gobject.TYPE_STRING
         if index == REVISION: return gobject.TYPE_PYOBJECT
         if index == PARENTS: return gobject.TYPE_PYOBJECT
-        if index == CHILDREN: return gobject.TYPE_PYOBJECT
+        if index == CHILDREN: return gobject.TYPE_PYOBJECT
+        if index == TAGS: return gobject.TYPE_PYOBJECT
         
     def on_get_iter(self, path):
         return path[0]
@@ -80,7 +83,15 @@
             return []
         if column == REVNO: return ".".join(["%d" % (revno)
                                       for revno in revno_sequence])
-        
+
+        if column == TAGS:
+            if not self.branch.supports_tags():
+                return []
+            try:
+                return self.branch.tags.get_reverse_tag_dict()[revid]
+            except KeyError:
+                return []
+
         if revid is None:
             return None
         if revid not in self.revisions:

=== modified file 'branchview/treeview.py'
--- branchview/treeview.py	2008-01-13 14:12:49 +0000
+++ branchview/treeview.py	2008-01-21 12:31:58 +0000
@@ -233,7 +233,7 @@
                                                         self.maxnum, 
                                                         broken_line_length)
 
-        self.model = TreeModel(self.branch.repository, linegraphdata)
+        self.model = TreeModel(self.branch, linegraphdata)
         self.graph_cell.columns_len = columns_len
         width = self.graph_cell.get_size(self.treeview)[2]
         if width > 500:
@@ -317,6 +317,7 @@
         self.graph_column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
         self.graph_column.pack_start(self.graph_cell, expand=False)
         self.graph_column.add_attribute(self.graph_cell, "node", treemodel.NODE)
+        self.graph_column.add_attribute(self.graph_cell, "tags", treemodel.TAGS)
         self.graph_column.add_attribute(self.graph_cell, "in-lines", treemodel.LAST_LINES)
         self.graph_column.add_attribute(self.graph_cell, "out-lines", treemodel.LINES)
         self.treeview.append_column(self.graph_column)

